<?php
// $Id: validation_api.admin.inc,v 1.6 2008/07/02 23:02:43 tapocol Exp $

/**
 * @file
 */

/**
 * Admin Page
 */
function validation_api_admin() {
  $links = array(
    l(t('Validators'), 'admin/build/validation_api/validators'),
    l(t('Fields'), 'admin/build/validation_api/fields')
  );
  
  return theme('item_list', $links);
}

/**
 * Lists all validators for the Admin Page
 */
function validation_api_admin_validators() {
  $validators = _validation_api_validators();
  
  $header = array(t('Name'), t('Type'), t('Message'), t('Ops'));
  $output = '';
  $rows = array();
  if (isset($validators) && is_array($validators)) {
    foreach ($validators as $key => $validator) {
      if (is_int($key)) {
        $rows[] = array($validator->name, $validator->type, $validator->message, l(t('edit'), 'admin/build/validation_api/validators/edit/'. $validator->vaid) .' '. l(t('delete'), 'admin/build/validation_api/validators/delete/'. $validator->vaid));
      }
      else {
        $rows[] = array($validator->name, $validator->type, $validator->message, t('from %key', array('%key' => $key)));
      }
    }
    
    $output .= theme('table', $header, $rows);
  }
  else {
    $output .= t('There are no validators in the database. !add or !import a validator.', array('!add' => l(t('Add'), 'admin/build/validation_api/validators/add'), '!import' => l(t('Import'), 'admin/build/validation_api/validators/import')));
  }
  
  return $output;
}

/**
 * The form for Adding or Editing Validators
 */
function validation_api_admin_validators_form($form_state, $vaid = NULL) {
  $defaults = array('name' => '', 'type' => '', 'rule' => '', 'message' => '');
  if (!is_null($vaid)) {
    drupal_set_breadcrumb(array_merge(drupal_get_breadcrumb(), array(l(t('Validators'), 'admin/build/validation_api/validators'))));
    if ($array = db_fetch_array(db_query('SELECT * FROM {validation_api} WHERE vaid = %d', $vaid))) {
      $defaults = $array;
    }
    else {
      drupal_not_found();
      exit;
    }
  }
  
  if (isset($defaults['vaid'])) {
    $form['#redirect'] = 'admin/build/validation_api/validators';
    $form['vaid'] = array(
      '#type' => 'value',
      '#value' => $defaults['vaid'],
    );
  }
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Unique name of the validator'),
    '#required' => TRUE,
    '#maxlength' => 16,
    '#default_value' => $defaults['name'],
  );
  $form['type'] = array(
    '#type' => 'textfield',
    '#title' => t('Type'),
    '#description' => t('The type of validator'),
    '#required' => TRUE,
    '#maxlength' => 16,
    '#default_value' => $defaults['type'],
  );
  $form['rule'] = array(
    '#type' => 'textarea',
    '#title' => t('Rule'),
    '#description' => t('Rule of the validator'),
    '#required' => TRUE,
    '#maxlength' => 256,
    '#rows' => 3,
    '#default_value' => $defaults['rule'],
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('The returned message on errors.'),
    '#required' => TRUE,
    '#maxlength' => 256,
    '#rows' => 3,
    '#default_value' => $defaults['message'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Validator'),
  );
  
  return $form;
}

/**
 * The Validate function for the Adding and Editing Validator Form
 */
function validation_api_admin_validators_form_validate($form_id, &$form_state) {
  $values = $form_state['values'];
  
  // Make sure the type field is a valid input
  switch ($values['type']) {
    case 'regex':
      // Make sure this is valid regex code
    break;
    case 'php':
      // Make sure this is valid PHP code
    break;
    default:
      // Send error when the type field does not use one of the above types
      form_set_error('type', 'Must be a valid type: regex or php');
  }
}

/**
 * The Submit function for the Adding and Editing Validator Form
 */
function validation_api_admin_validators_form_submit($form_id, &$form_state) {
  $values = $form_state['values'];
  
  // If vaid is present, then update it. Else, insert a new record
  if (isset($values['vaid'])) {
    // Run SQL query to update the vaid. Send message on success. Else, send error.
    if (db_query('UPDATE {validation_api} SET name = "%s", type = "%s", rule = "%s", message = "%s" WHERE vaid = %d', array($values['name'], $values['type'], $values['rule'], $values['message'], $values['vaid']))) {
      drupal_set_message(t('%name has been updated', array('%name' => $values['name'])));
    }
    else {
      drupal_set_message(t('%name could not be updated', array('%name' => $values['name'])), 'error');
    }
  }
  else {
    // Run SQL query to insert a new validator. Send message on success. Else, send error.
    if (db_query('INSERT INTO {validation_api} (name, type, rule, message) VALUES ("%s", "%s", "%s", "%s")', array($values['name'], $values['type'], $values['rule'], $values['message']))) {
      drupal_set_message(t('New Validator Added'));
    }
    else {
      drupal_set_message(t('Cound not add validator'), 'error');
    }
  }
}

/**
 * The form for Deleting Validators
 */
function validation_api_admin_validators_delete($form_state, $vaid) {
  if (!$obj = db_fetch_object(db_query('SELECT name FROM {validation_api} WHERE vaid = %d', $vaid))) {
    drupal_not_found();
    exit;
  }
  
  $form['#redirect'] = 'admin/build/validation_api/validators';
  $form['vaid'] = array(
    '#type' => 'value',
    '#value' => $vaid,
  );
  $form['name'] = array(
    '#type' => 'value',
    '#value' => $obj->name,
  );
  
  return confirm_form($form, t('Are you sure you want to delete the validator %name?', array('%name' => $obj->name)), 'admin/build/validation_api/validators', '', t('Delete'));
}

/**
 * The Submit function for the Validator Delete form
 */
function validation_api_admin_validators_delete_submit($form_id, &$form_state) {
  $values = $form_state['values'];
  
  if (db_query('DELETE FROM {validation_api} WHERE vaid = %d', $values['vaid'])) {
    drupal_set_message(t('%name was deleted', array('%name' => $values['name'])));
  }
  else {
    drupal_set_message(t('%name could not be deleted', array('%name' => $values['name'])));
  }
}

/**
 * Lists all fields for the Admin Page
 */
function validation_api_admin_fields() {
  $result = db_query('SELECT * FROM {validation_api_fields} ORDER BY form_id, form_field');
  $validators = _validation_api_validators();
  
  $header = array(t('Validator'), t('Argument'), t('Ops'));
  while ($obj = db_fetch_object($result)) {
    $forms[$obj->form_id][$obj->form_field][] = $obj;
  }
  
  $output = '';
  if (isset($forms) && is_array($forms)) {
    foreach ($forms as $form_id => $fields) {
      $rows = array();
      foreach ($fields as $field_name => $field) {
        foreach ($field as $validator) {
          $rows[] = array($validators[$validator->va_ref]->name, $validator->arg, l(t('edit'), 'admin/build/validation_api/fields/edit/'. $validator->vafid) .' '. l(t('delete'), 'admin/build/validation_api/fields/delete/'. $validator->vafid));
        }
        $output .= theme('table', $header, $rows, array(), $form_id .' - '. $field_name);
      }
    }
  }
  else {
    $output .= t('There are no fields being validated. !link.', array('!link' => l(t('Add a field'), 'admin/build/validation_api/fields/add')));
  }
  
  return $output;
}

/**
 * The form for Adding or Editing Fields
 */
function validation_api_admin_fields_form($form_state, $vafid = NULL) {
  $defaults = array('form_id' => '', 'form_field' => '', 'va_ref' => '', 'arg' => '');
  if (!is_null($vafid)) {
    drupal_set_breadcrumb(array_merge(drupal_get_breadcrumb(), array(l(t('Fields'), 'admin/build/validation_api/fields'))));
    if ($array = db_fetch_array(db_query('SELECT * FROM {validation_api_fields} WHERE vafid = %d', $vafid))) {
      $defaults = $array;
    }
    else {
      drupal_not_found();
      exit;
    }
  }
  
  $defaults['form_id'] = isset($_GET['form_id']) ? $_GET['form_id'] : $defaults['form_id'];
  $defaults['form_field'] = isset($_GET['form_field']) ? $_GET['form_field'] : $defaults['form_field'];
  
  if (isset($defaults['vafid'])) {
    $form['#redirect'] = 'admin/build/validation_api/fields';
    $form['vafid'] = array(
      '#type' => 'value',
      '#value' => $defaults['vafid'],
    );
  }
  $form['va_form_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Form ID'),
    '#description' => t('Function name that returns the structured form array.'),
    '#required' => TRUE,
    '#maxlength' => 32,
    '#default_value' => $defaults['form_id'],
  );
  $form['form_field'] = array(
    '#type' => 'textfield',
    '#title' => t('Field Name'),
    '#description' => t('The field element\'s name.'),
    '#required' => TRUE,
    '#maxlength' => 16,
    '#default_value' => $defaults['form_field'],
  );
  $validators = _validation_api_validators();
  foreach ($validators as $key => $validator) {
    $options[$key] = $validator->name;
  }
  $form['validator'] = array(
    '#type' => 'select',
    '#title' => t('Validator'),
    '#description' => t('The Validator to tie to the field.'),
    '#required' => TRUE,
    '#options' => $options,
    '#default_value' => $defaults['message'],
  );
  $form['arg'] = array(
    '#type' => 'textfield',
    '#title' => t('Argument'),
    '#description' => t('Argument to be passed with this validation.'),
    '#maxlength' => 16,
    '#default_value' => $defaults['arg'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Field'),
  );
  
  return $form;
}

/**
 * The Validate function for the Adding and Editing Field Form
 */
function validation_api_admin_fields_form_validate($form_id, &$form_state) {
  $values = $form_state['values'];
  
  // Check to see if the function exists during validation
  /*if (!function_exists($values['va_form_id']) && !is_array(call_user_func($values['va_form_id']))) {
    form_set_error('va_form_id', 'Form ID needs to be a function and it needs to return a structured form array.');
  }*/
}

/**
 * The Submit function for the Adding and Editing Field Form
 */
function validation_api_admin_fields_form_submit($form_id, &$form_state) {
  $values = $form_state['values'];
  
  // If vafid is present, then update it. Else, insert a new record
  if (isset($values['vafid'])) {
    // Run SQL query to update the vaid. Send message on success. Else, send error.
    if (db_query('UPDATE {validation_api_fields} SET form_id = "%s", form_field = "%s", va_ref = "%s", arg = "%s" WHERE vafid = %d', array($values['va_form_id'], $values['form_field'], $values['validator'], $values['arg'], $values['vafid']))) {
      drupal_set_message(t('%field has been updated', array('%field' => $values['form_field'])));
    }
    else {
      drupal_set_message(t('%field could not be updated', array('%field' => $values['form_field'])), 'error');
    }
  }
  else {
    // Run SQL query to insert a new validator. Send message on success. Else, send error.
    if (db_query('INSERT INTO {validation_api_fields} (form_id, form_field, va_ref, arg) VALUES ("%s", "%s", "%s", "%s")', array($values['va_form_id'], $values['form_field'], $values['validator'], $values['arg']))) {
      drupal_set_message(t('New Field Added'));
    }
    else {
      drupal_set_message(t('Cound not add field'), 'error');
    }
  }
}

/**
 * The form for Deleting Fields
 */
function validation_api_admin_fields_delete($form_state, $vafid) {
  if (!$obj = db_fetch_object(db_query('SELECT form_field FROM {validation_api_fields} WHERE vafid = %d', $vafid))) {
    drupal_not_found();
    exit;
  }
  
  $form['#redirect'] = 'admin/build/validation_api/fields';
  $form['vafid'] = array(
    '#type' => 'value',
    '#value' => $vafid,
  );
  $form['form_field'] = array(
    '#type' => 'value',
    '#value' => $obj->form_field,
  );
  
  return confirm_form($form, t('Are you sure you want to delete the validator %field?', array('%field' => $obj->form_field)), 'admin/build/validation_api/fields', '', t('Delete'));
}

/**
 * The Submit function for the Field Delete form
 */
function validation_api_admin_fields_delete_submit($form_id, &$form_state) {
  $values = $form_state['values'];
  
  if (db_query('DELETE FROM {validation_api_fields} WHERE vafid = %d', $values['vafid'])) {
    drupal_set_message(t('%field was deleted', array('%field' => $values['form_field'])));
  }
  else {
    drupal_set_message(t('%field could not be deleted', array('%field' => $values['form_field'])));
  }
}

function validation_api_admin_export_form() {
  $validators = _validation_api_validators();
  
  if (isset($validators) && is_array($validators)) {
    foreach ($validators as $key => $validator) {
      $opt_key = is_int($key) ? t('database') : $key;
      $options[$key] = t('%name from %key', array('%name' => $validator->name, '%key' => $opt_key));
    }
    
    $form['validators'] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Export',
    );
  }
  else {
    $form['validators'] = array(
      '#type' => 'item',
      '#value' => t('There are no validators in the database. !link.', array('!link' => l(t('Add a validator'), 'admin/build/validation_api/validators/add'))),
    );
  }
  
  return $form;
}

function validation_api_admin_export_form_submit($form_id, &$form_state) {
  echo _validation_api_export($form_state['values']['validators']);
  exit;
}

function validation_api_admin_import_form() {
  $validators = _validation_api_validators_from_hook();
  
  if (isset($validators) && is_array($validators)) {
    foreach ($validators as $key => $validator) {
      $options[$key] = t('%name from %key', array('%name' => $validator->name, '%key' => $key));
    }
    
    $form['validators'] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
    );
  }
  else {
    $form['validators'] = array(
      '#type' => 'item',
      '#value' => t('There are no validators in the database. !link.', array('!link' => l(t('Add a validator'), 'admin/build/validation_api/validators/add'))),
    );
  }
  
  $form['code'] = array(
    '#type' => 'textarea',
    '#title' => t('Import Code'),
    '#description' => t('Input the code is received from exporting validators.'),
    '#rows' => 15,
    '#default_value' => $defaults['message'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Import',
  );
  
  return $form;
}

function validation_api_admin_import_form_submit($form_id, &$form_state) {
  $validators = eval($form_state['values']['code']);
  if (isset($validators) && is_array($validators)) {
    foreach ($validators as $key => $validator) {
      if (isset($validator->module)) {
        unset($validators[$key]->module);
      }
      if (isset($validator->delta)) {
        unset($validators[$key]->delta);
      }
    }
  }
  if (isset($form_state['values']['validators']) && is_array($form_state['values']['validators'])) {
    foreach ($form_state['values']['validators'] as $key) {
      $array = explode('-', $key);
      $module = array_shift($array);
      $delta = (int) array_shift($array);
      if (!isset($return[$module])) {
        $return[$module] = module_invoke($module, HOOK_ADD_VALIDATOR);
      }
      $validators[$key] = $return[$module][$delta];
      $validators[$key]->module = $module;
      $validators[$key]->delta = $delta;
    }
  }
  if (isset($validators) && is_array($validators)) {
    _validation_api_import($validators);
  }
  else {
    drupal_set_message(t('Validator(s) were not returned in a proper format. Needs to be an array.'), 'error');
  }
}

function _validation_api_export($vaids) {
  $wheres = array();
  for ($i = 0; $i < count($vaids); $i++) {
    $wheres[] = 'vaid = %d';
  }
  $result = db_query('SELECT name, type, rule, message FROM {validation_api} WHERE '. implode(' OR ', $wheres), $vaids);
  
  $output = '';
  $i = 0;
  while ($validator = db_fetch_object($result)) {
    $output .= '$validators['. $i .']->name = \''. $validator->name ."';\n";
    $output .= '$validators['. $i .']->type = \''. $validator->type ."';\n";
    $output .= '$validators['. $i .']->rule = \''. $validator->rule ."';\n";
    $output .= '$validators['. $i++ .']->message = \''. $validator->message ."';\n";
  }
  $output .= 'return $validators;';
  
  return $output;
}

function _validation_api_import($validators) {
  $values = array();
  $args = array();
  foreach ($validators as $validator) {
    if (isset($validator->name) && isset($validator->type) && isset($validator->rule) && isset($validator->message)) {
      $str = '("%s", "%s", "%s", "%s"';
      array_push($args, $validator->name, $validator->type, $validator->rule, $validator->message);
      if (isset($validator->module) && isset($validator->delta)) {
        $str .= ', "%s", %d)';
        array_push($args, $validator->module, $validator->delta);
      }
      else {
        $str .= ', "", 0)';
      }
      $values[] = $str;
    }
  }
  if (count($values) > 0) {
    if (db_query('INSERT INTO {validation_api} (name, type, rule, message, module, delta) VALUES '. implode(', ', $values), $args)) {
      drupal_set_message(t('Successfully imported %count validator(s).', array('%count' => count($validators))));
    }
    else {
      drupal_set_message(t('Unsuccessful at importing validator(s).'), 'error');
    }
  }
  else {
    drupal_set_message(t('There were not any valid validators to be imported.'), 'error');
  }
}
