<?php
// $Id: validation_api.validators.inc,v 1.2 2008/05/22 19:23:54 tapocol Exp $

/**
 * @file
 */

define('VALIDATOR_LOWER', 'a-z');
define('VALIDATOR_UPPER', 'A-Z');
define('VALIDATOR_NUMERIC', '0-9');
define('VALIDATOR_ALPHA', 'a-zA-Z');
define('VALIDATOR_ALPHANUMERIC', 'a-zA-Z0-9');
define('VALIDATOR_HEXADECIMAL', 'a-fA-F0-9');

/**
 * Email Validator uses Drupal's valid_email_address function
 */
function validation_api_validators_email($value, $args) {
  return ((strlen($value) >= $args['min']) && ($args['max'] == 0 || strlen($value) <= $args['max'])) ? valid_email_address($value) : FALSE;
}

/**
 * URL Validator uses Drupal's valid_url function
 */
function validation_api_validators_url($value, $args) {
  return ((strlen($value) >= $args['min']) && ($args['max'] == 0 || strlen($value) <= $args['max'])) ? valid_url($value, $args['absolute']) : FALSE;
}

/**
 * Characters Validator
 */
function validation_api_validators_characters($value, $args) {
  $quantifier = validation_api_quantifier($args);
  $chars = '';
  
  foreach ($args['chars'] as $char) {
    if (is_array($char)) {
      if (isset($char['min']) && isset($char['max'])) {
        $chars .= $char['min'] .'-'. $char['max'];
      }
      else {
        watchdog('validator_error', 'Characters Validator $chars parameter has an array without associative keys of [min] and [max]. The form that is processing this error will never be able to validate the form.', $char, WATCHDOG_CRITICAL);
        return FALSE;
      }
    }
    elseif (is_string($char)) {
      if (defined('VALIDATOR_'. drupal_strtoupper($char))) {
        $chars .= constant('VALIDATOR_'. drupal_strtoupper($char));
      }
      else {
        $chars .= $char;
      }
    }
    else {
      watchdog('validator_error', 'Characters Validator $chars parameter has an array value(s) that is not an array or a string. The form that is processing this error will never be able to validate the form.', $char, WATCHDOG_CRITICAL);
      return FALSE;
    }
  }
  
  $pattern = '/^(['. $chars .']'. $quantifier .')$/';
  
  return preg_match($pattern, $value);
}

function validation_api_validators_selections($values, $args) {
  if (is_array($values)) {
    if (!(count($values) >= $args['min'] && ($args['max'] == 0 || count($values) <= $args['max']))) {
      return FALSE;
    }
  }
  else {
    watchdog('validator_error', 'Selections Validator $values parameter must be an array. Make sure you allow multiple selections for the form element. The form that is processing this error will never be able to validate the form.', $value, WATCHDOG_CRITICAL);
    return FALSE;
  }
  
  return TRUE;
}

/**
 * Custom Validator
 */
function validation_api_validators_custom($value, $args) {
  return preg_match($args['pattern'], $value);
}

/**
 * Create the quantifier notation in patterns from the $limits associative array
 */
function validation_api_quantifier($limits) {
  return ($limits['max'] == 0) ? '+' : '{'. $limits['min'] .','. $limits['max'] .'}';
}
